@@ -18,12 +18,13 @@ import sys
 from tempfile import NamedTemporaryFile
 import os
 from tornado.testing import AsyncTestCase, LogTrapTestCase
-from wstunnel.filters import DumpFilter
-from wstunnel.test import EchoServer, EchoClient
+from wstunnel.filters import DumpFilter, FilterException
+from wstunnel.test import EchoServer, EchoClient, RaiseFromWSFilter, RaiseToWSFilter
 from wstunnel.client import WSTunnelClient, WebSocketProxy
 from wstunnel.server import WSTunnelServer
 from wstunnel.toolbox import hex_dump, random_free_port
 
+
 __author__ = 'fabio'
 ASYNC_TIMEOUT = 2
 #TODO: on windows, temporary files are not working so well...
@@ -38,6 +39,8 @@ class WSEndpointsTestCase(AsyncTestCase, LogTrapTestCase):
 
     def no_response(self, response):
         self.stop()
+        self.fail("Connection should be dropped without a response if an error happens")
+
 
     def test_no_ws_endpoint(self):
         """
@@ -50,7 +53,14 @@ class WSEndpointsTestCase(AsyncTestCase, LogTrapTestCase):
         message = "Hello World!"
         client = EchoClient(clt_tun.address_list[0])
         client.send_message(message, self.no_response)
-        self.assertRaises(AssertionError, self.wait, timeout=ASYNC_TIMEOUT)
+        #self.wait(timeout=ASYNC_TIMEOUT)
+        with self.assertRaises(Exception):
+            try:
+                self.wait(timeout=ASYNC_TIMEOUT)
+            except Exception as e:
+                self.assertEqual(str(e),
+                                 "The server endpoint is not available, caused by HTTPError('HTTP 599: [Errno 61] Connection refused',))")
+                raise e
 
     def test_no_server_service(self):
         """
@@ -67,7 +77,23 @@ class WSEndpointsTestCase(AsyncTestCase, LogTrapTestCase):
         message = "Hello World!"
         client = EchoClient(clt_tun.address_list[0])
         client.send_message(message, self.no_response)
-        self.assertRaises(AssertionError, self.wait, timeout=ASYNC_TIMEOUT)
+        with self.assertRaises(Exception):
+            try:
+                self.wait(timeout=ASYNC_TIMEOUT)
+            except Exception as e:
+                #print(e)
+                #self.assertEqual(str(e),)
+                raise e
+
+    def test_no_client_ws_options(self):
+        """
+        Tests the client tunnel endpoint behaviour when there's no server counterpart
+        """
+        clt_tun = WSTunnelClient(family=socket.AF_INET,
+                                 io_loop=self.io_loop)
+        clt_tun.add_proxy("test", WebSocketProxy(port=0, ws_url="ws://localhost:{0}/test".format(random_free_port())))
+        clt_tun.start()
+        self.assertEqual(clt_tun.ws_options, {})
 
 
 class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
@@ -94,6 +120,13 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
         self.message = "Hello World!".encode("utf-8")
         self.client = EchoClient(self.clt_tun.address_list[0])
 
+    def tearDown(self):
+
+        for srv in self.srv, self.srv_tun, self.clt_tun:
+            srv.stop()
+
+        super(WSTunnelTestCase, self).tearDown()
+
     def on_response_received(self, response):
         """
         Callback invoked when response is received
@@ -151,6 +184,28 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
 
             self.srv_tun.uninstall_filter(server_filter)
 
+    def test_raise_filter_exception_from_ws(self):
+        """
+        Tests the behaviour when a filter raises exception reading from websocket
+        """
+        server_filter = RaiseFromWSFilter()
+        self.srv_tun.install_filter(server_filter)
+
+        self.client.send_message(self.message, self.on_response_received)
+        #TODO: this generically catch assertion errors... We should get the inner FilterException in some way
+        self.assertRaises(AssertionError, self.wait, timeout=1)
+
+    def test_raise_filter_exception_to_ws(self):
+        """
+        Tests the behaviour when a filter raises exception writing to websocket
+        """
+        server_filter = RaiseToWSFilter()
+        self.srv_tun.install_filter(server_filter)
+
+        self.client.send_message(self.message, self.on_response_received)
+        #TODO: this generically catch assertion errors... We should get the inner FilterException in some way
+        self.assertRaises(AssertionError, self.wait, timeout=1)
+
     def test_add_get_remove_proxy(self):
         """
         Tests adding/remove/get operations
@@ -163,13 +218,6 @@ class WSTunnelTestCase(AsyncTestCase, LogTrapTestCase):
         self.clt_tun.remove_proxy("test_add_remove")
         self.assertFalse(ws_proxy.serving)
 
-    def tearDown(self):
-
-        for srv in self.srv, self.srv_tun, self.clt_tun:
-            srv.stop()
-
-        super(WSTunnelTestCase, self).tearDown()
-
 
 class WSTunnelSSLTestCase(WSTunnelTestCase):
     """
